'use strict';
var chai = require('chai');
var ZSchema = require('z-schema');
var customFormats = module.exports = function(zSchema) {
  // Placeholder file for all custom-formats in known to swagger.json
  // as found on
  // https://github.com/OAI/OpenAPI-Specification/blob/master/versions/2.0.md#dataTypeFormat

  var decimalPattern = /^\d{0,8}.?\d{0,4}[0]+$/;

  /** Validates floating point as decimal / money (i.e: 12345678.123400..) */
  zSchema.registerFormat('double', function(val) {
    return !decimalPattern.test(val.toString());
  });

  /** Validates value is a 32bit integer */
  zSchema.registerFormat('int32', function(val) {
    // the 32bit shift (>>) truncates any bits beyond max of 32
    return Number.isInteger(val) && ((val >> 0) === val);
  });

  zSchema.registerFormat('int64', function(val) {
    return Number.isInteger(val);
  });

  zSchema.registerFormat('float', function(val) {
    // better parsing for custom "float" format
    if (Number.parseFloat(val)) {
      return true;
    } else {
      return false;
    }
  });

  zSchema.registerFormat('date', function(val) {
    // should parse a a date
    return !isNaN(Date.parse(val));
  });

  zSchema.registerFormat('dateTime', function(val) {
    return !isNaN(Date.parse(val));
  });

  zSchema.registerFormat('password', function(val) {
    // should parse as a string
    return typeof val === 'string';
  });
};

customFormats(ZSchema);

var validator = new ZSchema({});
var request = require('request');
var expect = chai.expect;

describe('/divisione_ok', function() {
  describe('get', function() {
    it('should respond with 200 OK! and should work with all required properties', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "number"
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        qs: {
          a: -4839474.238726631,b: 34757000.365471005
        },
        method: 'GET',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(200);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });

    it('should respond with 400 Numbers are missing and should not work with type \'boolean\' of a', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "code": {
            "type": "number"
          },
          "message": {
            "type": "string"
          }
        },
        "required": [
          "code",
          "message"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        qs: {
          a: false,b: 34757000.365471005
        },
        method: 'GET',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(400);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });
  it('should respond with 400 Numbers are missing and should not work with type \'null\' of a', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "code": {
            "type": "number"
          },
          "message": {
            "type": "string"
          }
        },
        "required": [
          "code",
          "message"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        qs: {
          a: null,b: 34757000.365471005
        },
        method: 'GET',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(400);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });
  it('should respond with 400 Numbers are missing and should not work with type \'string\' of a', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "code": {
            "type": "number"
          },
          "message": {
            "type": "string"
          }
        },
        "required": [
          "code",
          "message"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        qs: {
          a: 'e]gC8o[',b: 34757000.365471005
        },
        method: 'GET',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(400);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });
  it('should respond with 400 Numbers are missing and should not work with type \'boolean\' of b', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "code": {
            "type": "number"
          },
          "message": {
            "type": "string"
          }
        },
        "required": [
          "code",
          "message"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        qs: {
          a: -4839474.238726631,b: true
        },
        method: 'GET',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(400);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });
  it('should respond with 400 Numbers are missing and should not work with type \'null\' of b', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "code": {
            "type": "number"
          },
          "message": {
            "type": "string"
          }
        },
        "required": [
          "code",
          "message"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        qs: {
          a: -4839474.238726631,b: null
        },
        method: 'GET',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(400);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });
  it('should respond with 400 Numbers are missing and should not work with type \'string\' of b', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "object",
        "properties": {
          "code": {
            "type": "number"
          },
          "message": {
            "type": "string"
          }
        },
        "required": [
          "code",
          "message"
        ]
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        qs: {
          a: -4839474.238726631,b: 'LugBnJaPE'
        },
        method: 'GET',
        headers: {
          'Content-Type': 'application/json'
        }
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(400);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });

  
  });

  describe('post', function() {
    it('should respond with 200 ok! and should work with all required properties', function(done) {
      /*eslint-disable*/
      var schema = {
        "type": "number"
      };

      /*eslint-enable*/
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"a":54416404.07834929,"b":92665777.25740853}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect(res.statusCode).to.equal(200);

		expect(validator.validate(body, schema)).to.be.true;
        done();
      });
    });

  
    it('should respond with default undefined and should not work without required property: a', function(done) {
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"b":-32693045.798083812}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work without required property: b', function(done) {
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"a":-64274700.92440388}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'a\' of type \'boolean\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"a":true,"b":89835092.0006406}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'a\' of type \'null\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"a":null,"b":-47041982.627544954}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'a\' of type \'string\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"a":"e)Mq[NUX&0","b":65483437.360840976}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'b\' of type \'boolean\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"a":87039264.24725434,"b":false}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'b\' of type \'null\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"a":-13116655.045742482,"b":null}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });
  it('should respond with default undefined and should not work with \'b\' of type \'string\'', function(done) {
      request({
        url: 'http://localhost:8080/v1/divisione_ok',
        json: true,
        method: 'POST',
        headers: {
          'Content-Type': 'application/json'
        },
        body: {"a":817268.8416136652,"b":"XFZOs"}
      },
      function(error, res, body) {
        if (error) return done(error);

        expect([200,400]).to.include(res.statusCode)

		// expect(body).to.equal(null); // non-json response or no schema
        done();
      });
    });

  });

});
